@@ -27,8 +27,9 @@ import (
 )
 
 type Config struct {
-	Images   *Images               `json:"-"`
-	Platform configv1.PlatformType `json:"-"`
+	Images      *Images               `json:"-"`
+	RemoteWrite bool                  `json:"-"`
+	Platform    configv1.PlatformType `json:"-"`
 
 	PrometheusOperatorConfig             *PrometheusOperatorConfig `json:"prometheusOperator"`
 	PrometheusOperatorUserWorkloadConfig *PrometheusOperatorConfig `json:"prometheusOperatorUserWorkload"`
@@ -85,6 +86,7 @@ type PrometheusK8sConfig struct {
 	VolumeClaimTemplate *v1.PersistentVolumeClaim `json:"volumeClaimTemplate"`
 	Hostport            string                    `json:"hostport"`
 	RemoteWrite         []monv1.RemoteWriteSpec   `json:"remoteWrite"`
+	TelemetryMatches    []string                  `json:"-"`
 }
 
 type AlertmanagerMainConfig struct {
@@ -217,7 +219,9 @@ func (c *Config) applyDefaults() {
 		c.HTTPConfig = &HTTPConfig{}
 	}
 	if c.TelemeterClientConfig == nil {
-		c.TelemeterClientConfig = &TelemeterClientConfig{}
+		c.TelemeterClientConfig = &TelemeterClientConfig{
+			TelemeterServerURL: "https://infogw.api.openshift.com/",
+		}
 	}
 	if c.K8sPrometheusAdapter == nil {
 		c.K8sPrometheusAdapter = &K8sPrometheusAdapter{}
@@ -248,6 +252,17 @@ func (c *Config) SetImages(images map[string]string) {
 	c.Images.Thanos = images["thanos"]
 }
 
+func (c *Config) SetTelemetryMatches(matches []string) {
+	c.PrometheusK8sConfig.TelemetryMatches = matches
+}
+
+func (c *Config) SetRemoteWrite(rw bool) {
+	c.RemoteWrite = rw
+	if c.RemoteWrite && c.TelemeterClientConfig.TelemeterServerURL == "https://infogw.api.openshift.com/" {
+		c.TelemeterClientConfig.TelemeterServerURL = "https://infogw.api.openshift.com/metrics/v1/receive"
+	}
+}
+
 func (c *Config) LoadClusterID(load func() (*configv1.ClusterVersion, error)) error {
 	if c.TelemeterClientConfig.ClusterID != "" {
 		return nil
