@@ -33,16 +33,23 @@ import (
 	configv1 "github.com/openshift/api/config/v1"
 	routev1 "github.com/openshift/api/route/v1"
 	securityv1 "github.com/openshift/api/security/v1"
+	"github.com/openshift/cluster-monitoring-operator/pkg/promqlgen"
 	"github.com/pkg/errors"
 	appsv1 "k8s.io/api/apps/v1"
 	v1 "k8s.io/api/core/v1"
 	"k8s.io/api/extensions/v1beta1"
 	rbacv1 "k8s.io/api/rbac/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
+	"k8s.io/apimachinery/pkg/util/intstr"
 	"k8s.io/apimachinery/pkg/util/yaml"
 	apiregistrationv1beta1 "k8s.io/kube-aggregator/pkg/apis/apiregistration/v1beta1"
 )
 
+const (
+	configManagedNamespace = "openshift-config-managed"
+	sharedConfigMap        = "monitoring-shared-config"
+)
+
 var (
 	AlertmanagerConfig             = "assets/alertmanager/secret.yaml"
 	AlertmanagerService            = "assets/alertmanager/service.yaml"
@@ -195,6 +202,9 @@ var (
 	AuthProxyExternalURLFlag  = "-external-url="
 	AuthProxyCookieDomainFlag = "-cookie-domain="
 	AuthProxyRedirectURLFlag  = "-redirect-url="
+
+	PrometheusTrustedCABundleDir  = "/etc/pki/prometheus-ca-bundle/"
+	PrometheusTrustedCABundlePath = PrometheusTrustedCABundleDir + "ca-bundle.crt"
 )
 
 const (
@@ -355,43 +365,38 @@ func (f *Factory) AlertmanagerMain(host string, trustedCABundleCM *v1.ConfigMap)
 		a.Spec.Tolerations = f.config.AlertmanagerMainConfig.Tolerations
 	}
 
-	setEnv := func(container *v1.Container, name, value string) {
-		for i := range container.Env {
-			if container.Env[i].Name == name {
-				container.Env[i].Value = value
+	a.Spec.Containers[0].Image = f.config.Images.OauthProxy
+	setEnv := func(name, value string) {
+		for i := range a.Spec.Containers[0].Env {
+			if a.Spec.Containers[0].Env[i].Name == name {
+				a.Spec.Containers[0].Env[i].Value = value
+				break
 			}
 		}
 	}
-	for i, c := range a.Spec.Containers {
-		switch c.Name {
-		case "alertmanager-proxy":
-			a.Spec.Containers[i].Image = f.config.Images.OauthProxy
-			if f.config.HTTPConfig.HTTPProxy != "" {
-				setEnv(&a.Spec.Containers[i], "HTTP_PROXY", f.config.HTTPConfig.HTTPProxy)
-			}
-			if f.config.HTTPConfig.HTTPSProxy != "" {
-				setEnv(&a.Spec.Containers[i], "HTTPS_PROXY", f.config.HTTPConfig.HTTPSProxy)
-			}
-			if f.config.HTTPConfig.NoProxy != "" {
-				setEnv(&a.Spec.Containers[i], "NO_PROXY", f.config.HTTPConfig.NoProxy)
-			}
+	if f.config.HTTPConfig.HTTPProxy != "" {
+		setEnv("HTTP_PROXY", f.config.HTTPConfig.HTTPProxy)
+	}
+	if f.config.HTTPConfig.HTTPSProxy != "" {
+		setEnv("HTTPS_PROXY", f.config.HTTPConfig.HTTPSProxy)
+	}
+	if f.config.HTTPConfig.NoProxy != "" {
+		setEnv("NO_PROXY", f.config.HTTPConfig.NoProxy)
+	}
 
-			if trustedCABundleCM != nil {
-				volumeName := "alertmanager-trusted-ca-bundle"
-				a.Spec.VolumeMounts = append(a.Spec.VolumeMounts, trustedCABundleVolumeMount(volumeName))
-				volume := trustedCABundleVolume(trustedCABundleCM.Name, volumeName)
-				volume.VolumeSource.ConfigMap.Items = append(volume.VolumeSource.ConfigMap.Items, v1.KeyToPath{
-					Key:  "ca-bundle.crt",
-					Path: "tls-ca-bundle.pem",
-				})
-				a.Spec.Volumes = append(a.Spec.Volumes, volume)
-				a.Spec.Containers[i].VolumeMounts = append(
-					a.Spec.Containers[i].VolumeMounts,
-					trustedCABundleVolumeMount(volumeName),
-				)
-			}
-		}
+	if trustedCABundleCM != nil {
+		volumeName := "alertmanager-trusted-ca-bundle"
+		a.Spec.VolumeMounts = append(a.Spec.VolumeMounts, trustedCABundleVolumeMount(volumeName))
+		volume := trustedCABundleVolume(trustedCABundleCM.Name, volumeName)
+		volume.VolumeSource.ConfigMap.Items = append(volume.VolumeSource.ConfigMap.Items, v1.KeyToPath{
+			Key:  "ca-bundle.crt",
+			Path: "tls-ca-bundle.pem",
+		})
+		a.Spec.Volumes = append(a.Spec.Volumes, volume)
+		// We have only one container in Alertmanager CR spec and this is oauth-proxy
+		a.Spec.Containers[0].VolumeMounts = append(a.Spec.Containers[0].VolumeMounts, trustedCABundleVolumeMount(volumeName))
 	}
+
 	a.Namespace = f.namespace
 
 	return a, nil
@@ -1064,7 +1069,7 @@ func (f *Factory) ThanosQuerierRoute() (*routev1.Route, error) {
 	return r, nil
 }
 
-func (f *Factory) SharingConfig(promHost, amHost, grafanaHost, thanosHost *url.URL) *v1.ConfigMap {
+func (f *Factory) SharingConfigDeprecated(promHost, amHost, grafanaHost, thanosHost *url.URL) *v1.ConfigMap {
 	return &v1.ConfigMap{
 		ObjectMeta: metav1.ObjectMeta{
 			Name:      "sharing-config",
@@ -1079,6 +1084,23 @@ func (f *Factory) SharingConfig(promHost, amHost, grafanaHost, thanosHost *url.U
 	}
 }
 
+func (f *Factory) SharingConfig(promHost, amHost, grafanaHost, thanosHost *url.URL) *v1.ConfigMap {
+	return &v1.ConfigMap{
+		ObjectMeta: metav1.ObjectMeta{
+			Name:      sharedConfigMap,
+			Namespace: configManagedNamespace,
+		},
+		Data: map[string]string{
+			// Configmap keys need to include "public" to indicate that they are public values.
+			// See https://bugzilla.redhat.com/show_bug.cgi?id=1807100.
+			"grafanaPublicURL":      grafanaHost.String(),
+			"prometheusPublicURL":   promHost.String(),
+			"alertmanagerPublicURL": amHost.String(),
+			"thanosPublicURL":       thanosHost.String(),
+		},
+	}
+}
+
 func (f *Factory) PrometheusK8sTrustedCABundle() (*v1.ConfigMap, error) {
 	cm, err := f.NewConfigMap(MustAssetReader(PrometheusK8sTrustedCABundle))
 	if err != nil {
@@ -1133,8 +1155,69 @@ func (f *Factory) PrometheusK8s(host string, grpcTLS *v1.Secret, trustedCABundle
 		}
 	}
 
+	telemetryEnabled := f.config.TelemeterClientConfig.IsEnabled()
+	if telemetryEnabled && f.config.RemoteWrite {
+
+		selectorRelabelConfig, err := promqlgen.LabelSelectorsToRelabelConfig(f.config.PrometheusK8sConfig.TelemetryMatches)
+		if err != nil {
+			return nil, errors.Wrap(err, "generate label selector relabel config")
+		}
+
+		compositeToken, err := json.Marshal(map[string]string{
+			"cluster_id":          f.config.TelemeterClientConfig.ClusterID,
+			"authorization_token": f.config.TelemeterClientConfig.Token,
+		})
+
+		spec := monv1.RemoteWriteSpec{
+			URL:         f.config.TelemeterClientConfig.TelemeterServerURL,
+			BearerToken: base64.StdEncoding.EncodeToString(compositeToken),
+			QueueConfig: &monv1.QueueConfig{
+				Capacity: 10000,
+				// Default: 100
+				MaxSamplesPerSend: 1000,
+				// Default: 1000
+				MaxShards: 100,
+				// Default: 30ms
+				MinBackoff: "500ms",
+				// Default: 100ms
+				MaxBackoff: "2s",
+			},
+			WriteRelabelConfigs: []monv1.RelabelConfig{
+				*selectorRelabelConfig,
+				monv1.RelabelConfig{
+					TargetLabel: "_id",
+					Replacement: f.config.TelemeterClientConfig.ClusterID,
+				},
+				// relabeling the `ALERTS` series to `alerts` allows us to make
+				// a distinction between the series produced in-cluster and out
+				// of cluster.
+				monv1.RelabelConfig{
+					SourceLabels: []string{"__name__"},
+					TargetLabel:  "__name__",
+					Regex:        "ALERTS",
+					Replacement:  "alerts",
+				},
+			},
+		}
+
+		p.Spec.RemoteWrite = []monv1.RemoteWriteSpec{spec}
+
+	}
+	if !telemetryEnabled {
+		p.Spec.RemoteWrite = nil
+	}
+
 	if len(f.config.PrometheusK8sConfig.RemoteWrite) > 0 {
-		p.Spec.RemoteWrite = f.config.PrometheusK8sConfig.RemoteWrite
+		p.Spec.RemoteWrite = append(p.Spec.RemoteWrite, f.config.PrometheusK8sConfig.RemoteWrite...)
+	}
+
+	for _, rw := range p.Spec.RemoteWrite {
+		if f.config.HTTPConfig.HTTPProxy != "" {
+			rw.ProxyURL = f.config.HTTPConfig.HTTPProxy
+		}
+		if f.config.HTTPConfig.HTTPSProxy != "" {
+			rw.ProxyURL = f.config.HTTPConfig.HTTPSProxy
+		}
 	}
 
 	if !f.config.EtcdConfig.IsEnabled() {
@@ -2227,6 +2310,33 @@ func (f *Factory) NewPrometheusRule(manifest io.Reader) (*monv1.PrometheusRule,
 	return p, nil
 }
 
+func (f *Factory) NewTelemeterPrometheusRecRuleFromString(expr string) (*monv1.PrometheusRule, error) {
+	p := &monv1.PrometheusRule{
+		ObjectMeta: metav1.ObjectMeta{
+			Name: "telemetry",
+		},
+		Spec: monv1.PrometheusRuleSpec{
+			Groups: []monv1.RuleGroup{
+				{
+					Name: "telemeter.rules",
+					Rules: []monv1.Rule{
+						{
+							Record: "cluster:telemetry_selected_series:count",
+							Expr:   intstr.FromString(expr),
+						},
+					},
+				},
+			},
+		},
+	}
+
+	if p.GetNamespace() == "" {
+		p.SetNamespace(f.namespace)
+	}
+
+	return p, nil
+}
+
 func (f *Factory) NewAlertmanager(manifest io.Reader) (*monv1.Alertmanager, error) {
 	a, err := NewAlertmanager(manifest)
 	if err != nil {
@@ -2492,10 +2602,17 @@ func (f *Factory) TelemeterClientDeployment(proxyCABundleCM *v1.ConfigMap) (*app
 	d.Spec.Template.Spec.Containers[1].Image = f.config.Images.ConfigmapReloader
 	d.Spec.Template.Spec.Containers[2].Image = f.config.Images.KubeRbacProxy
 
-	d.Spec.Template.Spec.Containers[0].Command = append(
-		d.Spec.Template.Spec.Containers[0].Command,
-		"--limit-bytes=5242880",
-	)
+	cmd := []string{}
+	for _, a := range d.Spec.Template.Spec.Containers[0].Command {
+		if !strings.HasPrefix(a, "--match=") {
+			cmd = append(cmd, a)
+		}
+	}
+	for _, m := range f.config.PrometheusK8sConfig.TelemetryMatches {
+		cmd = append(cmd, fmt.Sprintf("--match=%s", m))
+	}
+	cmd = append(cmd, "--limit-bytes=5242880")
+	d.Spec.Template.Spec.Containers[0].Command = cmd
 
 	if len(f.config.TelemeterClientConfig.NodeSelector) > 0 {
 		d.Spec.Template.Spec.NodeSelector = f.config.TelemeterClientConfig.NodeSelector
